--[[
GemJar
v. 0.3.1

Author: Camaris  of EU-Kul Tiras

Disclaimer: I am not responsible if your computer explodes after using this addon.
]]--

local TradeSkillFrame_Update_ORIGINAL
local ATSW = false
local GEMJAR_DEBUG = false
local loaded = false
local font

local function GemJar_Debug(str)
	if GEMJAR_DEBUG then
		DEFAULT_CHAT_FRAME:AddMessage("[GemJar]: "..str,1,1,0,1)
	end
end

GemJar_Debug("Started.")

-- Item Levels of uncut gems
local LK_EPIC 				= 110 -- guesstimate
local LK_BLUE				= 80
local LK_GREEN			= 75
local TBC_EPIC			= 100
local TBC_BLUE			= 70
local TBC_GREEN		= 65

local function GemJar_isGem(skillIndex,skillName)
	return string.find(skillName, GEMJAR_DRAGONSEYE) -- For some reason, Dragon's Eyes have itemfamily 0 
							or (GetItemFamily(GetTradeSkillItemLink(skillIndex)) == 512)
end

local function GemJar_isBoPGem(gem)
	local isBoP = false
	for _,pattern in pairs(GEMJAR_BOP_GEMS) do
		isBoP = isBoP or string.find(gem,pattern)
	end
	return isBoP
end

local function GemJar_SetGemColor(skillIndex,skillName,skillType)
	_,_,_,font = TradeSkillTypeColor[skillType] -- Defined in Blizzard_TradeskillUI
	if skillType ~= 'header' and GemJar_isGem(skillIndex,skillName) then
		local uncutGem = GetTradeSkillReagentItemLink(skillIndex,1)
		if (uncutGem) then
			local _,_,_,gemLevel,_ = GetItemInfo(uncutGem)
			local _,_,itemRarity,_ = GetItemInfo(GetTradeSkillItemLink(skillIndex))
			if itemRarity == 2 then -- Green
				if gemLevel == LK_GREEN then font = GemJarFontGreen else font = GemJarFontGreenFaded end
			elseif itemRarity == 3 then -- Blue
				if gemLevel == LK_BLUE then font = GemJarFontBlue else font = GemJarFontBlueFaded end
			elseif itemRarity == 4 then -- Epic
				if gemLevel == LK_EPIC then font = GemJarFontEpic else font = GemJarFontEpicFaded end
			end
			if GemJar_isBoPGem(skillName) then
				if gemLevel == LK_EPIC or gemLevel == LK_BLUE or gemLEvel == LK_GREEN then font = GemJarFontBoP else font = GemJarFontBoPFaded end
			end
		else
			-- This can happen when the reagent is not in local cache
			GemJar_Debug("Cannot find reagent data for "..skillName)
		end
	end
end

function GemJar_TradeSkillFrame_Update ()
	TradeSkillFrame_Update_ORIGINAL()
	-- the following is basically stolen from the existing TradeSkillFrame_Update, but rewrites the coloring bit
	local skillName, skillType, skillIndex, skillButton
	local skillOffset = FauxScrollFrame_GetOffset(TradeSkillListScrollFrame)
	for i=1, TRADE_SKILLS_DISPLAYED, 1 do
		skillIndex = i + skillOffset
		skillName, skillType,_ = GetTradeSkillInfo(skillIndex)
		skillButton = _G['TradeSkillSkill'..i]
		GemJar_SetGemColor(skillIndex,skillName,skillType)
		if (font) then
			skillButton:SetNormalFontObject(font);
		end
	end
end

function GemJar_ATSWFrame_Update()
	TradeSkillFrame_Update_ORIGINAL()
	
	local skillName, skillType, skillIndex, skillButton
	local numTradeSkills, skillOffset
	local skillOffset=FauxScrollFrame_GetOffset(ATSWListScrollFrame)
	local jumped = 1
		
	--*****************************************************--
	if(atsw_orderby[UnitName("player")][atsw_selectedskill]=="custom") then
		GemJar_Debug("ASWT Custom sorting NYI")
	elseif (atsw_orderby[UnitName("player")][atsw_selectedskill]=="nothing") then
		numTradeSkills=table.getn(atsw_skilllisting);
		for i=1, ATSW_TRADE_SKILLS_DISPLAYED, 1 do
			repeat
				skillIndex=skillOffset+jumped;
				if(skillIndex>numTradeSkills) then
					skillName=nil;
				else
					skillName = atsw_skilllisting[skillIndex].name;
					skillType = atsw_skilllisting[skillIndex].type;
				end
				jumped=jumped+1;
			until ((skillName and ATSW_Filter(skillName)==true and ATSW_FilterInvSlot(skillName) and ATSW_FilterSubClass(skillName)) or skillIndex>numTradeSkills or skillType=="header")
			----------------------
			if(skillIndex<=numTradeSkills) then
				skillButton = _G['ATSWSkill'..i]
				GemJar_SetGemColor(skillIndex,skillName,skillType)
				if (font) then
					skillButton:SetNormalFontObject(font);
				end
			end
			----------------------
		end
	--*****************************************************--	
	elseif (atsw_orderby[UnitName("player")][atsw_selectedskill]=="name" or atsw_orderby[UnitName("player")][atsw_selectedskill]=="difficulty") then
		numTradeSkills=table.getn(atsw_tradeskillid);
		for i=1, ATSW_TRADE_SKILLS_DISPLAYED, 1 do
			repeat
				skillIndex=atsw_tradeskillid[skillOffset+jumped];
				if(skillIndex==nil) then
					skillName=nil;
				else	
					skillName, skillType, _, _ = GetTradeSkillInfo(skillIndex);
				end
				jumped=jumped+1;
			until (((skillName and ATSW_Filter(skillName)==true and ATSW_FilterInvSlot(skillName) and ATSW_FilterSubClass(skillName)) or skillIndex==nil) and skillType~="header");
			----------------------
			if(skillIndex) then
				skillButton = _G['ATSWSkill'..i]
				GemJar_SetGemColor(skillIndex,skillName,skillType)
				if (font) then
					skillButton:SetNormalFontObject(font);
				end
			end
			----------------------
		end
	end
end

local function GemJar_Load()
	if ATSW then
		-- ATSW, hook their function
		TradeSkillFrame_Update_ORIGINAL = ATSWFrame_Update
		ATSWFrame_Update = GemJar_ATSWFrame_Update
	else
		-- No ATSW, hook Blizzard function
		TradeSkillFrame_Update_ORIGINAL = TradeSkillFrame_Update
		TradeSkillFrame_Update = GemJar_TradeSkillFrame_Update
	end
	loaded = true
	GemJar_Debug("Hooked.")
end

local function GemJar_Unload()
	if (TradeSkillFrame_Update_ORIGINAL) then
		if ATSW then
			ATSWFrame_Update = TradeSkillFrame_Update_ORIGINAL
		else
			TradeSkillFrame_Update = TradeSkillFrame_Update_ORIGINAL
		end
		loaded = false
		GemJar_Debug("Unhooked.")
	else
		GemJar_Debug("Unhooking failed.")
	end
end


function GemJar_OnEvent(self, event, ...)
	if event == "TRADE_SKILL_SHOW" then
		local curProfession,_ = GetTradeSkillLine()
		if curProfession == GEMJAR_JEWELCRAFTING and not loaded then
			GemJar_Load()
		end
	elseif event == "TRADE_SKILL_CLOSE" then
		if loaded then
			GemJar_Unload()
		end
	elseif event == "ADDON_LOADED" then
		if not ATSW and IsAddOnLoaded("AdvancedTradeSkillWindow") then
			GemJar_Debug("ATSW Detected")
			ATSW = true
		end
	end
end